@@ -1,10 +1,10 @@
 
-from rusty_axe.tree_reader_utils import fast_knn, double_fast_knn, hacked_louvain, generate_feature_value_html,sister_distance
-from rusty_axe.tree_reader_node import Node, Reduction, Filter
-from rusty_axe.tree_reader_prediction import Prediction
-from rusty_axe.tree_reader_sample_cluster import SampleCluster
-from rusty_axe.tree_reader_node_cluster import NodeCluster
-from rusty_axe.tree_reader_tree import Tree
+from rusty_axe.utils import fast_knn, double_fast_knn, hacked_louvain, generate_feature_value_html, sister_distance
+from rusty_axe.node import Node, Reduction, Filter
+from rusty_axe.prediction import Prediction
+from rusty_axe.sample_cluster import SampleCluster
+from rusty_axe.node_cluster import NodeCluster
+from rusty_axe.tree import Tree
 
 from json import dumps as jsn_dumps
 from os import makedirs
@@ -79,7 +79,7 @@ class Forest:
             node.index = i
 
         if split_labels is not None:
-            self.external_split_labels(self.nodes(),split_labels,roots=True)
+            self.external_split_labels(self.nodes(), split_labels, roots=True)
 
 ########################################################################
 ########################################################################
@@ -89,11 +89,10 @@ class Forest:
 ########################################################################
 ########################################################################
 
-
     # A method that allows one to summon a multiprocessing pool general to the forest
 
     def pool(self):
-        if hasattr(self,'pool'):
+        if hasattr(self, 'pool_object'):
             if self.pool_object is not None:
                 return self.pool_object
 
@@ -129,11 +128,11 @@ class Forest:
 
     def reindex_nodes(self):
         nodes = self.nodes()
-        for i,node in enumerate(nodes):
+        for i, node in enumerate(nodes):
             node.index = i
 
-    def reindex_samples(self,samples):
-        map = {s:i for i,s in list(enumerate(samples))}
+    def reindex_samples(self, samples):
+        map = {s: i for i, s in list(enumerate(samples))}
         for node in self.nodes():
             if node.local_samples is not None:
                 new_samples = [map[s] for s in node.local_samples]
@@ -162,10 +161,10 @@ class Forest:
     def roots(self):
         return [tree.root for tree in self.trees]
 
-    def trim(self,limit):
+    def trim(self, limit):
 
-        for i,tree in enumerate(self.trees):
-            print(f"Trimming {i}\r",end='')
+        for i, tree in enumerate(self.trees):
+            print(f"Trimming {i}\r", end='')
             tree.trim(limit)
         print("")
 
@@ -177,25 +176,27 @@ class Forest:
         leaf_mask[[leaf.index for leaf in self.leaves()]] = True
         return leaf_mask
 
-########################################################################
-########################################################################
+    """
 
-# NODE/MATRIX METHODS
+    NODE/MATRIX METHODS
 
-# Methods for turning a set of nodes into an encoding matrix
+    Methods for turning a set of nodes into an encoding matrix
 
-# Encoding matrices are boolean matrices, usually node x property
-# Sample encoding matrix would be i,j, where i is ith node and j is whether
-# sample j appears in that node
+    Encoding matrices are matrices, usually node x property
+    Sample encoding matrix would be i,j, where i is ith node and j is whether
+    sample j appears in that node
 
-########################################################################
-########################################################################
+    """
 
-
-    def node_representation(self, nodes=None, mode='additive_mean', metric=None, pca=0):
+    def node_representation(self, nodes=None, mode='partial', metric=None, pca=0):
         from sklearn.decomposition import IncrementalPCA
 
-        # ROWS ARE NODES, COLUMNS ARE WHATEVER
+        """
+        Generic node representation method, wraps several concrete implementations, switches via the "mode" argument
+        Calls out to other methods for representation, then does optional post-processing (PCA) or conversion to a metric.
+
+        If metric is not none, the representation returns a metric instead of a raw representation. Metrics are called out to scipy.
+        """
 
         if nodes is None:
             nodes = self.nodes()
@@ -218,7 +219,7 @@ class Forest:
             encoding = self.mean_matrix(nodes)
         elif mode == 'factor':
             encoding = self.node_factor_encoding(nodes)
-        elif mode == 'partial':
+        elif mode == 'partial' or mode == "partials":
             encoding = self.partial_matrix(nodes)
         elif mode == 'partial_absolute':
             encoding = self.partial_absolute(nodes)
@@ -229,7 +230,6 @@ class Forest:
             if pca > encoding.shape[1]:
                 print("WARNING, PCA DIMENSION TOO SMALL, PICKING MINIMUM")
                 pca = np.min([pca, encoding.shape[1]])
-            # print(f"debug:{encoding.shape}")
             from sklearn.decomposition import IncrementalPCA
             model = IncrementalPCA(n_components=pca)
             chunks = int(np.floor(encoding.shape[0] / 10000)) + 1
@@ -238,29 +238,21 @@ class Forest:
                 print(f"Learning chunk {i}\r", end='')
                 model.partial_fit(encoding[(i - 1) * 10000:i * 10000])
             model.partial_fit(encoding[-last_chunk:])
-            # transformed = model.transform(encoding)
-            # print(f"Chunks:{chunks}")
             transformed = np.zeros((encoding.shape[0], pca))
             for i in range(1, chunks):
                 print(f"Transforming chunk {i}\r", end='')
-                # print(f"coordinates:{((i-1)*10000,i*10000)}")
                 transformed[(
                     i - 1) * 10000:i * 10000] = model.transform(encoding[(i - 1) * 10000:i * 10000])
-            # print(f"coordinates:{-last_chunk}")
             transformed[-last_chunk:] = model.transform(encoding[-last_chunk:])
             print("")
             encoding = transformed
-            # encoding = PCA(n_components=pca).fit_transform(encoding)
-
-            # encoding = PCA(n_components=pca).fit_transform(encoding)
 
         if metric == "sister":
             if mode != "sister":
                 raise Exception(f"Mode and metric mismatched {mode},{metric}")
             else:
                 representation = sister_distance(encoding)
-
-        if metric is not None:
+        elif metric is not None:
             representation = squareform(pdist(encoding, metric=metric))
         else:
             representation = encoding
@@ -284,17 +276,16 @@ class Forest:
             encoding[:, i] = node.encoding()
         return encoding
 
-    def node_factor_encoding(self,nodes):
+    def node_factor_encoding(self, nodes):
 
-        encoding = np.zeros((len(nodes),len(self.split_clusters)),dtype=bool)
+        encoding = np.zeros((len(nodes), len(self.split_clusters)), dtype=bool)
 
-        for i,factor in enumerate(self.split_clusters):
+        for i, factor in enumerate(self.split_clusters):
             mask = factor.node_mask()
-            encoding[:,i][mask] = True
+            encoding[:, i][mask] = True
 
         return encoding
 
-
     def node_sister_encoding(self, nodes):
         encoding = np.zeros((len(self.samples), len(nodes)), dtype=int)
         for i, node in enumerate(nodes):
@@ -333,7 +324,7 @@ class Forest:
             gains[:, i] = node.additive_mean_gains()
         return gains
 
-    def conditional_gain_matrix(self,nodes):
+    def conditional_gain_matrix(self, nodes):
         return self.mean_additive_matrix(nodes)
 
     def mean_matrix(self, nodes):
@@ -345,6 +336,8 @@ class Forest:
     def median_matrix(self, nodes):
         predictions = np.zeros((len(nodes), len(self.output_features)))
         for i, node in enumerate(nodes):
+            if i % 10 == 0:
+                print(f"Estimating node {i}\r", end='')
             predictions[i] = node.medians()
         return predictions
 
@@ -354,20 +347,20 @@ class Forest:
             weights[i] = node.weights
         return weights
 
-    def partial_matrix(self,nodes):
-        partials = np.zeros((len(nodes),len(self.output_features)))
+    def partial_matrix(self, nodes):
+        partials = np.zeros((len(nodes), len(self.output_features)))
         for i, node in enumerate(nodes):
-            if i%1000 == 0:
-                print(f"Node {i}\r",end='')
+            if i % 1000 == 0:
+                print(f"Node {i}\r", end='')
             partials[i] = node.partials()
         print("")
         return partials
 
-    def partial_absolute(self,nodes):
-        partials = np.zeros((len(nodes),len(self.output_features)))
+    def partial_absolute(self, nodes):
+        partials = np.zeros((len(nodes), len(self.output_features)))
         for i, node in enumerate(nodes):
-            if i%1000 == 0:
-                print(f"Node {i}\r",end='')
+            if i % 1000 == 0:
+                print(f"Node {i}\r", end='')
             partials[i] = node.absolute_partials()
         print("")
         return partials
@@ -379,6 +372,78 @@ class Forest:
             weighted_predictions[i] = node.weighted_prediction_cache
         return weighted_predictions
 
+    """
+    Factor Summary Methods:
+
+    Methods that summarize the factors, usually in matrix form.
+    """
+
+
+
+    def factor_partial_matrix(self, features=None):
+        if features is None:
+            coordinates = np.zeros(
+                (len(self.split_clusters), len(self.output_features)))
+            for i, split_cluster in enumerate(self.split_clusters):
+                coordinates[i] = np.mean(
+                    self.partial_matrix(split_cluster.nodes), axis=1)
+        else:
+            coordinates = np.zeros((len(self.split_clusters), len(features)))
+            for i, split_cluster in enumerate(self.split_clusters):
+                for j, feature in enumerate(features):
+                    coordinates[i, j] = split_cluster.feature_partial(
+                        feature)
+        return coordinates
+
+    def factor_feature_matrix(self, features=None):
+
+        """
+        Average additive gain across all nodes in all factors, in array form.
+
+        Returns: Factor X Feature numpy array, where rows are factors, columns are features
+        """
+
+        if features is None:
+            coordinates = np.zeros(
+                (len(self.split_clusters), len(self.output_features)))
+            for i, split_cluster in enumerate(self.split_clusters):
+                coordinates[i] = np.mean(
+                    self.mean_additive_matrix(split_cluster.nodes), axis=1)
+        else:
+            coordinates = np.zeros((len(self.split_clusters), len(features)))
+            for i, split_cluster in enumerate(self.split_clusters):
+                for j, feature in enumerate(features):
+                    coordinates[i, j] = split_cluster.feature_mean_additive(
+                        feature)
+        return coordinates
+
+    def factor_mean_matrix(self, features=None):
+
+        """
+        Arguments:
+        features = Selects a small subset of features to look at instead of all features.
+        If None, all features are returned. If a list of indices, then only means of those features are returned, in the order they are passed in.
+
+        Mean across the means of all nodes in a factor.
+        Ok for looking at the identity of samples in the factor.
+
+        Returns: Factor x Feature numpy array. Rows are factors, columns are features.
+        """
+
+        if features is None:
+            coordinates = np.zeros(
+                (len(self.split_clusters), len(self.output_features)))
+            for i, split_cluster in enumerate(self.split_clusters):
+                coordinates[i] = np.mean(
+                    self.mean_matrix(split_cluster.nodes), axis=0)
+        else:
+            coordinates = np.zeros((len(self.split_clusters), len(features)))
+            for i, split_cluster in enumerate(self.split_clusters):
+                for j, feature in enumerate(features):
+                    coordinates[i, j] = split_cluster.feature_mean(feature)
+        return coordinates
+
+
 ########################################################################
 ########################################################################
 
@@ -428,7 +493,7 @@ class Forest:
 
         trees = []
         for tree_file in combined_tree_files:
-            print(f"Loading {tree_file}")
+            print(f"Loading {tree_file}\r", end='')
             trees.append(
                 Tree(json.load(open(tree_file.strip())), first_forest))
 
@@ -449,18 +514,19 @@ class Forest:
 
     def from_sklearn(forest):
 
-
         raw_trees = [e.tree_ for e in forest.estimators_]
 
         trees = []
 
-        def node_recursion(index,children_left,children_right):
+        def node_recursion(index, children_left, children_right):
             nodes = []
             left_child = children_left[index]
             right_child = children_right[index]
             if left_child > 0 and right_child > 0:
-                nodes.extend(node_recursion(left_child,children_left,children_right))
-                nodes.extend(node_recursion(right_child,children_left,children_right))
+                nodes.extend(node_recursion(
+                    left_child, children_left, children_right))
+                nodes.extend(node_recursion(
+                    right_child, children_left, children_right))
                 nodes.append(left_child)
                 nodes.append(right_child)
             return nodes
@@ -468,43 +534,16 @@ class Forest:
         for raw_tree in raw_trees:
             children_left = raw_tree.children_left
             children_right = raw_tree.children_right
-            nodes = node_recursion(0,children_left,children_right)
+            nodes = node_recursion(0, children_left, children_right)
             trees.append(nodes)
 
         return trees
 
-
-    def from_sklearn(forest):
-
-
-        raw_trees = [e.tree_ for e in forest.estimators_]
-
-        trees = []
-
-        def node_recursion(index,children_left,children_right):
-            nodes = []
-            left_child = children_left[index]
-            right_child = children_right[index]
-            if left_child > 0 and right_child > 0:
-                nodes.extend(node_recursion(left_child,children_left,children_right))
-                nodes.extend(node_recursion(right_child,children_left,children_right))
-                nodes.append(left_child)
-                nodes.append(right_child)
-            return nodes
-
-        for raw_tree in raw_trees:
-            children_left = raw_tree.children_left
-            children_right = raw_tree.children_right
-            nodes = node_recursion(0,children_left,children_right)
-            trees.append(nodes)
-
-        return trees
-
-    def derive_samples(self,samples):
+    def derive_samples(self, samples):
 
         new_trees = []
 
-        for i,tree in enumerate(self.trees):
+        for i, tree in enumerate(self.trees):
             print(f"Deriving tree {i}")
             new_trees.append(tree.derive_samples(samples))
 
@@ -512,8 +551,8 @@ class Forest:
             new_trees,
             copy.deepcopy(self.input[samples]),
             copy.deepcopy(self.output[samples]),
-            input_features = copy.deepcopy(self.input_features),
-            output_features = copy.deepcopy(self.output_features),
+            input_features=copy.deepcopy(self.input_features),
+            output_features=copy.deepcopy(self.output_features),
             samples=list(np.array(self.samples)[samples]),
             split_labels=None,
             cache=self.cache
@@ -526,7 +565,8 @@ class Forest:
             node.forest = new_forest
 
         split_labels = [n.split_cluster for n in new_forest.nodes()]
-        new_forest.external_split_labels(new_forest.nodes(),split_labels,roots=True)
+        new_forest.external_split_labels(
+            new_forest.nodes(), split_labels, roots=True)
 
         new_forest.reindex_samples(samples)
 
@@ -534,15 +574,14 @@ class Forest:
 
         return new_forest
 
-
     def set_cache(self, value):
         self.cache = value
         for node in self.nodes():
             node.cache = value
 
     def compute_cache(self):
-        for i,tree in enumerate(self.trees):
-            print(f"Tree {i}",end='\r')
+        for i, tree in enumerate(self.trees):
+            print(f"Tree {i}", end='\r')
             tree.root.compute_cache()
         print("")
 
@@ -601,72 +640,11 @@ class Forest:
                 for feature in removed_features:
                     self.remove_output_feature(feature)
 
-########################################################################
-########################################################################
-
-# PREDICTION METHODS
-
-# This section deals with methods that allow predictions on
-# samples
-
-########################################################################
-########################################################################
 
     def predict(self, matrix):
         prediction = Prediction(self, matrix)
         return prediction
 
-    def predict_sample_leaves(self, sample):
-        sample_leaves = []
-        for tree in self.trees:
-            sample_leaves.extend(tree.root.predict_sample_leaves(sample))
-        return sample_leaves
-
-    def predict_sample_nodes(self, sample):
-        sample_nodes = []
-        for tree in self.trees:
-            sample_nodes.extend(tree.root.predict_sample_nodes(sample))
-        return sample_nodes
-
-    def predict_vector_leaves(self, vector, features=None):
-        # if features is None:
-        # features = self.input_features
-        sample = {feature: value for feature,
-                  value in zip(range(len(vector)), vector)}
-        return self.predict_sample_leaves(sample)
-
-    def predict_vector_nodes(self, vector, features=None):
-        # if features is None:
-        #     features = self.input_features
-        sample = {feature: value for feature,
-                  value in zip(range(len(vector)), vector)}
-        return self.predict_sample_nodes(sample)
-
-    def predict_node_sample_encoding(self, matrix, leaves=True, depth=None):
-        encodings = []
-        for i, root in enumerate(self.roots()):
-            print(f"Predicting tree:{i}\r", end='')
-            encodings.append(root.predict_matrix_encoding(matrix))
-            encodings.append(np.ones(matrix.shape[0], dtype=bool))
-        print('')
-        encoding = np.vstack(encodings)
-        if leaves:
-            encoding = encoding[self.leaf_mask()]
-        if depth is not None:
-            depth_mask = np.zeros(encoding.shape[0], dtype=bool)
-            for n in self.nodes():
-                if n.level <= depth:
-                    depth_mask[n.index] = True
-            encoding = encoding[depth_mask]
-        return encoding
-
-########################################################################
-########################################################################
-
-# CLUSTERING METHODS
-
-########################################################################
-########################################################################
 
     def split_labels(self, depth=3):
 
@@ -703,7 +681,6 @@ class Forest:
         else:
             if pca is not None:
                 counts = PCA(n_components=pca).fit_transform(counts)
-                # self.set_sample_labels(sdg.fit_predict(counts, *args, **kwargs))
                 self.set_sample_labels(hacked_louvain(
                     fast_knn(counts, **kwargs), resolution=resolution))
             else:
@@ -731,7 +708,6 @@ class Forest:
         if hasattr(self, 'sample_clusters') and not override:
             print("Clustering has already been done")
         else:
-            # self.set_sample_labels(sdg.fit_predict(encoding, *args, **kwargs))
             self.set_sample_labels(
                 1 + hacked_louvain(fast_knn(encoding, **kwargs), resolution=resolution))
 
@@ -750,8 +726,6 @@ class Forest:
 
         print(f"encoding dimensions: {encoding.shape}")
 
-        # from scipy.stats import mode
-
         sample_labels = []
 
         for leaf_mask in encoding:
@@ -760,8 +734,6 @@ class Forest:
             odds = leaf_cluster_counts / leaf_cluster_sizes
             sample_labels.append(np.argmax(odds))
 
-        # sample_labels = [mode(leaf_clusters[mask])[0][0] for mask in encoding]
-
         self.set_sample_labels(np.array(sample_labels).astype(dtype=int))
 
         return self.sample_labels
@@ -784,22 +756,6 @@ class Forest:
         self.leaf_clusters = clusters
         for leaf, label in zip(leaves, self.leaf_labels):
             leaf.leaf_cluster = label
-    #
-    # def cluster_leaves_samples(self, override=False, pca=None, depth=None, *args, **kwargs):
-    #
-    #     leaves = self.leaves(depth=depth)
-    #     encoding = self.node_sample_encoding(leaves).T
-    #
-    #     if pca is not None:
-    #         encoding = PCA(n_components=pca).fit_transform(encoding)
-    #
-    #     if hasattr(self, 'leaf_clusters') and not override:
-    #         print("Clustering has already been done")
-    #         return self.leaf_labels
-    #     else:
-    #         self.set_leaf_labels(sdg.fit_predict(encoding, *args, **kwargs))
-    #
-    #     return self.leaf_labels
 
     def cluster_leaves_predictions(self, override=False, mode='mean', *args, **kwargs):
 
@@ -882,7 +838,7 @@ class Forest:
 
         return ordered_features, ordered_coefficients
 
-    def interpret_splits(self, override=False, mode='partial', metric='cosine', pca=100, relatives=True, resolution=1, k=10, depth=6, **kwargs):
+    def interpret_splits(self, override=False, mode='partial', metric='cosine', pca=100, relatives=True, resolution=1, k=100, depth=6, **kwargs):
 
         if pca > len(self.output_features):
             print(
@@ -898,18 +854,11 @@ class Forest:
 
         if relatives:
 
-            # print("Relativistic distance (heh)")
-
             own_representation = self.node_representation(
                 nodes[stem_mask], mode=mode, pca=pca)
             sister_representation = self.node_representation(
                 [n.sister() for n in nodes[stem_mask]], mode=mode, pca=pca)
 
-            # print("Representations:")
-            # print(own_representation.shape)
-            # print(sister_representation.shape)
-
-            # print("Running double knn")
             knn = double_fast_knn(own_representation,
                                   sister_representation, k=k, metric=metric, **kwargs)
 
@@ -917,15 +866,12 @@ class Forest:
             representation = self.node_representation(
                 nodes[stem_mask], mode=mode, pca=pca)
 
-            # print("Running knn")
             knn = fast_knn(representation, k=k, metric=metric, **kwargs)
 
-        # print("Calling clustering procedure")
         labels[stem_mask] = 1 + hacked_louvain(knn, resolution=resolution)
 
         for node, label in zip(nodes, labels):
             node.set_split_cluster(label)
-            # node.split_cluster = label
 
         cluster_set = set(labels)
         clusters = []
@@ -934,12 +880,11 @@ class Forest:
             clusters.append(NodeCluster(
                 self, [nodes[i] for i in split_index], cluster))
 
-        # split_order = np.argsort(labels)
-        # split_order = dendrogram(linkage(reduction,metric='cos',method='average'),no_plot=True)['leaves']
-
         self.split_clusters = clusters
         self.factors = self.split_clusters
 
+        self.maximum_spanning_tree(mode='samples')
+
         return labels
 
     def external_split_labels(self, nodes, labels, roots=False):
@@ -949,6 +894,9 @@ class Forest:
         cluster_set = set(labels)
         clusters = []
 
+        if not roots:
+            labels = [l+1 for l in labels]
+
         for node, label in zip(nodes, labels):
             node.set_split_cluster(label)
             # node.split_cluster = label
@@ -963,9 +911,6 @@ class Forest:
             clusters.append(NodeCluster(
                 self, [nodes[i] for i in split_index], cluster))
 
-        # split_order = np.argsort(labels)
-        # split_order = dendrogram(linkage(reduction,metric='cos',method='average'),no_plot=True)['leaves']
-
         self.split_clusters = clusters
 
     def create_root_cluster(self):
@@ -1047,9 +992,6 @@ class Forest:
         highlight = np.ones(combined_coordinates.shape[0]) * 10
         highlight[len(self.sample_labels):] = [len(cluster.samples)
                                                for cluster in self.sample_clusters]
-        # for i,cluster in enumerate(self.sample_clusters):
-        #
-        #     highlight[self.counts.shape[0] + i:] = len(cluster.samples/10)
 
         combined_labels = np.zeros(
             self.output.shape[0] + len(self.sample_clusters))
@@ -1069,7 +1011,6 @@ class Forest:
             for cluster, coordinates in zip(cluster_names, cluster_coordiantes):
                 plt.text(*coordinates, cluster, verticalalignment='center',
                          horizontalalignment='center')
-            # plt.savefig("./tmp.delete.png",dpi=300)
         else:
             f = plt.figure(figsize=(20, 20))
             plt.title("Sample Coordinates")
@@ -1134,8 +1075,6 @@ class Forest:
                 representation.T, metric=metric, method='average'), no_plot=True)['leaves']
             image = representation[agg_f].T[agg_s].T
         else:
-            # feature_order = dendrogram(linkage(reduction.T+1,metric='cosine',method='average'),no_plot=True)['leaves']
-            # image = reduction[split_order].T[feature_order].T
             try:
                 agg_f = dendrogram(linkage(
                     representation.T, metric='cosine', method='average'), no_plot=True)['leaves']
@@ -1176,50 +1115,6 @@ class Forest:
         return coordinates
 
 
-    def factor_partial_matrix(self, features=None):
-        if features is None:
-            coordinates = np.zeros(
-                (len(self.split_clusters), len(self.output_features)))
-            for i, split_cluster in enumerate(self.split_clusters):
-                coordinates[i] = np.mean(
-                    self.partial_matrix(split_cluster.nodes), axis=1)
-        else:
-            coordinates = np.zeros((len(self.split_clusters), len(features)))
-            for i, split_cluster in enumerate(self.split_clusters):
-                for j, feature in enumerate(features):
-                    coordinates[i, j] = split_cluster.feature_partial(
-                        feature)
-        return coordinates
-
-    def factor_feature_matrix(self, features=None):
-        if features is None:
-            coordinates = np.zeros(
-                (len(self.split_clusters), len(self.output_features)))
-            for i, split_cluster in enumerate(self.split_clusters):
-                coordinates[i] = np.mean(
-                    self.mean_additive_matrix(split_cluster.nodes), axis=1)
-        else:
-            coordinates = np.zeros((len(self.split_clusters), len(features)))
-            for i, split_cluster in enumerate(self.split_clusters):
-                for j, feature in enumerate(features):
-                    coordinates[i, j] = split_cluster.feature_mean_additive(
-                        feature)
-        return coordinates
-
-    def factor_mean_matrix(self, features=None):
-        if features is None:
-            coordinates = np.zeros(
-                (len(self.split_clusters), len(self.output_features)))
-            for i, split_cluster in enumerate(self.split_clusters):
-                coordinates[i] = np.mean(
-                    self.mean_matrix(split_cluster.nodes), axis=0)
-        else:
-            coordinates = np.zeros((len(self.split_clusters), len(features)))
-            for i, split_cluster in enumerate(self.split_clusters):
-                for j, feature in enumerate(features):
-                    coordinates[i, j] = split_cluster.feature_mean(feature)
-        return coordinates
-
     def tsne(self, no_plot=False, pca=100, override=False, **kwargs):
         if not hasattr(self, 'tsne_coordinates') or override:
             if pca:
@@ -1297,14 +1192,16 @@ class Forest:
 
         return self.umap_coordinates
 
-    def coordinates(self, type=None, scaled=True, **kwargs):
+    def precomputed_coordinates(self):
+        return self.precomputed_coordinate_cache
+
+    def coordinates(self, override=False, type=None, scaled=True, **kwargs):
+
+        if hasattr(self,'coordinate_cache') and not override:
+            return self.coordinate_cache
 
         if type is None:
-            if hasattr(self, 'coordinate_type'):
-                type = self.coordinate_type
-            else:
-                type = 'tsne'
-
+            type = 'umap'
         self.coordinate_type = type
 
         type_functions = {
@@ -1322,41 +1219,35 @@ class Forest:
 
         return coordinates
 
+    def set_coordiantes(self,coordinates):
+        self.coordinate_type = 'precomputed'
+        self.coordinate_cache = coordinates
+
     def plot_manifold(self, depth=3):
 
         f = self.plot_sample_clusters()
 
         def recursive_tree_plot(parent, children, figure, level=0):
-            # print("Recursion debug")
-            # print(f"p:{parent}")
-            # print(f"c:{children}")
             pc = self.split_clusters[parent].coordinates()
             vectors = []
             for child, sub_children in children:
                 if child == len(self.split_clusters):
                     continue
                 cc = self.split_clusters[child].coordinates()
-                # print("coordinates")
-                # print(f"p{parent}:{pc}")
-                # print(f"c{child}:{cc}")
-                # v = pc - cc
                 v = cc - pc
-                # print(f"v:{v}")
                 plt.figure(figure.number)
-                # plt.arrow(pc[0],pc[1],v[0],v[1],width=(depth+1-level)*.3,length_includes_head=True)
                 plt.arrow(pc[0], pc[1], v[0], v[1], length_includes_head=True)
                 vectors.append((pc, v))
                 figure, cv = recursive_tree_plot(
                     child, sub_children, figure, level=level + 1)
                 vectors.extend(cv)
-            # print("===============")
             return figure, vectors
 
         f, v = recursive_tree_plot(self.likely_tree[0], self.likely_tree[1], f)
-        #
-        # plt.savefig("./tmp.delete.png",dpi=300)
-        #
-        # return f,v
+
+        plt.savefig("./tmp.delete.png", dpi=300)
+
+        return f, v
 
     def plot_braid_vectors(self):
 
@@ -1379,6 +1270,7 @@ class Forest:
 ########################################################################
 ########################################################################
 
+
     def split_cluster_transition_matrix(self, depth=3):
 
         nodes = np.array(self.nodes(depth=depth))
@@ -1429,28 +1321,7 @@ class Forest:
 
         return upstream_frequency, downstream_frequency
 
-    # def conditional_split_probability(self):
-    #
-    #     _, down_matrix = self.directional_matrix()
-    #     total_descendents = np.sum(down_matrix, axis=1)
-    #     conditional_probability = (down_matrix.T / (total_descendents + 1)).T
-    #
-    #     return conditional_probability
-
-    # def probability_enrichment(self):
-    #
-    #     _, down_matrix = self.directional_matrix()
-    #     total_descendents = np.sum(down_matrix, axis=1)
-    #     conditional_probability = (down_matrix.T / (total_descendents + 1)).T
-    #
-    #     raw_probability = conditional_probability[0]
-    #     enrichment = conditional_probability / raw_probability
-    #
-    #     return enrichment
-
-
-
-    def path_matrix(self,nodes=None):
+    def path_matrix(self, nodes=None):
 
         if nodes is None:
             nodes = self.nodes()
@@ -1459,8 +1330,6 @@ class Forest:
         for node in nodes:
             if hasattr(node, 'split_cluster'):
                 mtx[node.split_cluster, node.index] = True
-                # for descendant in node.nodes():
-                #     mtx[node.split_cluster, descendant.index] = True
 
         return mtx
 
@@ -1468,7 +1337,6 @@ class Forest:
         path_matrix = self.path_matrix()
         path_covariance = np.cov(path_matrix)
         precision = np.linalg.pinv(path_covariance)
-    #     return precision
 
         precision_normalization = np.sqrt(
             np.outer(np.diag(precision), np.diag(precision)))
@@ -1593,22 +1461,16 @@ class Forest:
             mean_matrix = self.split_cluster_mean_matrix()
             domain_matrix = self.split_cluster_domain_mean_matrix()
             distances = 1. - cdist(mean_matrix, domain_matrix, metric="cosine")
-            # distances = squareform(1. - pdist(mean_matrix,domain_matrix,metric="cosine"))
         elif mode == "samples":
             cluster_values = np.array([c.sample_scores()
                                        for c in self.split_clusters])
-            parent_values = np.array([c.parent_scores() for c in self.split_clusters])
-            # cluster_values = np.array([np.abs(c.sister_scores())
-            #                            for c in self.split_clusters])
-            # parent_values = np.array([c.parent_scores() for c in self.split_clusters])
+            parent_values = np.array([c.parent_scores()
+                                      for c in self.split_clusters])
 
-            normed_cv = cluster_values / np.sum(cluster_values,axis=0)
-            normed_pv = parent_values / np.sum(parent_values,axis=0)
-            distances = 1. - cdist(normed_cv,normed_pv,metric='cosine')
+            normed_cv = cluster_values / np.sum(cluster_values, axis=0)
+            normed_pv = parent_values / np.sum(parent_values, axis=0)
+            distances = 1. - cdist(normed_cv, normed_pv, metric='cosine')
 
-            # distances = 1. - cdist(parent_values,cluster_values,metric='cosine')
-            # distances = 1. - cdist(cluster_values,sister_values,metric='cosine')
-            # distances = squareform(1. - pdist(cluster_values, metric="cosine"))
         else:
             raise Exception(f"Not a valid mode: {mode}")
 
@@ -1619,13 +1481,7 @@ class Forest:
 
         clusters = set(range(len(self.split_clusters)))
 
-        print("Max tree debug")
-        print(distances)
-        print(mst)
-
         def finite_tree(cluster, available):
-            # print(cluster)
-            # print(mst[cluster])
             children = []
             try:
                 available.remove(cluster)
@@ -1650,6 +1506,7 @@ class Forest:
 # HTML Visualization methods
 #########################################################
 
+
     def html_directory(self):
 
         location = Path(__file__).parent.absolute()
@@ -1662,26 +1519,33 @@ class Forest:
 
     def html_tree_summary(self, n=3, mode="ud", custom=None, labels=None, features=None, primary=True, cmap='viridis', secondary=True, figsize=(30, 30), output=None):
 
-        # First we'd like to make sure we are operating from scratch in the html directory:
 
         if n > (self.output.shape[1] / 2):
             print("WARNING, PICKED N THAT IS TOO LARGE, SETTING LOWER")
             n = max(int(self.output.shape[1] / 2), 1)
 
+
+        # First we'd like to make sure we are operating from scratch in the html directory:
         if output is None:
-            # tmp_dir = tmp.TemporaryDirectory()
-            # html_location = location = tmp_dir.name + "/"
             location = self.location()
             html_location = self.html_directory()
-            rmtree(html_location)
+            try:
+                rmtree(html_location)
+            except FileNotFoundError:
+                pass
             makedirs(html_location)
         else:
             location = self.location()
             html_location = output
 
+        cluster_jsons = []
+
         for split_cluster in self.split_clusters:
             print(f"Summarizing:{split_cluster.name()}")
-            split_cluster.html_cluster_summary(n=n, plot=False, output=html_location+str(split_cluster.id)+"/")
+            cj = split_cluster.json_cluster_summary(n=n)
+            cluster_jsons.append(cj)
+            split_cluster.html_cluster_summary(
+                n=n, plot=False, output=html_location + str(split_cluster.id) + "/", json=cj)
 
         copyfile(location + "/tree_template.html",
                  html_location + "tree_template.html")
@@ -1723,15 +1587,10 @@ class Forest:
                     child_width += cw
                 if child_width == 0:
                     child_width = width
-                # print("Recursive tree debug")
-                # print(f"x:{x},y:{y}")
-                # print(f"{tree[0]}")
-                # print(f"cw:{child_width}")
 
                 # We have to place the current leaf at the average position of all leaves below
                 padding = (child_width - width) / 2
                 coordinates = [x + padding + (width * .5), y + (height * .5)]
-                # print(f"coordinates:{coordinates}")
 
                 child_coordinates.append([int(tree[0]), coordinates])
 
@@ -1826,12 +1685,6 @@ class Forest:
                             child_center_x = cx + (width / 2)
                             child_center_y = cy + (height / 2)
 
-                            # And plot a line with a weight equivalent to the number of transitions
-
-                            # cp = self.split_cluster_transitions[i,j]
-                            # total = np.sum(self.split_cluster_transitions[i])
-                            # arrow_canvas.plot([center_x,child_center_x],[center_y,child_center_y],alpha=min(1,cp/total*2),linewidth=(cp**2)*.01,transform=arrow_canvas.transAxes)
-
                             # Alternatively, plot a line with a weight equivalent to the partial correlation of split clusters:
 
                             cp = self.dependence_scores[i, j]
@@ -1850,16 +1703,16 @@ class Forest:
 
             # Now we need to loop over available clusters to place the cluster decorations into the template
 
-            for cluster in self.split_clusters:
-                cluster_summary_json = cluster.json_cluster_summary(n=n)
-                cluster_summary_html = f'<script> summaries["cluster_{cluster.id}"] = {cluster_summary_json};</script>'
+            for i,cj in enumerate(cluster_jsons):
+                cluster_summary_html = f"<script> summaries['cluster_{i}'] = {cj};</script>"
                 html_report.write(cluster_summary_html)
 
         from subprocess import run
         try:
             run(["open", html_location + "tree_template.html"])
         except:
-            print(f"Stored the html report at {html_location}, but could not open from command line")
+            print(
+                f"Stored the html report at {html_location}, but could not open from command line")
 
     def split_cluster_leaves(self):
         def tree_leaves(tree):
@@ -1899,8 +1752,7 @@ class Forest:
             matrix[:, i] = cluster.sister_scores()
         return matrix
 
-
-    def global_correlations(self,indices=None):
+    def global_correlations(self, indices=None):
 
         if indices is None:
             indices = np.arange(self.output.shape[1])
@@ -1910,6 +1762,7 @@ class Forest:
         return correlations
 
 
+
 class TruthDictionary:
 
     def __init__(self, counts, header, samples=None):
